/*
VS1010 SetDate

SetDate sets the date and/or time given as parameter to the RTC.

Ported from VS1005. In addition to rtc and time, mktime.c and the ParamSpl
program are required.
*/
#include <vo_stdio.h>
#include <volink.h>     // Linker directives like DLLENTRY
#include <string.h>
#include <vs1010bRom.h>
#include "time.h"
#include "rtc.h"


// Lenghts of months for normal and leap years.
int __mem_y monLen[2][12] = {
	{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
	  {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
};

ioresult main(char *parameters) {
	
	// Split parameters and return their amount
	int nParam = RunLib("ParamSpl",parameters);
	char *p = parameters;
	int i;

	// The set and tim variables are used to check if both date and time 
	// have been set. time() returns a negative value to tim if neither have
	// been set yet. 
	int set = 0;
	time_t tim = time(NULL);
	
	// tm is used to temporarily hold the set time.
	struct tm *tm = localtime(&tim);
	
	ioresult res = S_OK;
	
	// RunLib returns the value -31234 if ParamSpl isn't found. Because 
	// 0 > -31234 the for loop will be skipped and because the set variable 
	// is initalized to 0, the latter if statement will be skipped as well.
	if (nParam == -31234) {
		printf("ParamSpl not found\n");
	}

	// Loop through the given parameters
	for (i=0; i<nParam; i++) {
	
		// Print the help and quit the program. Time and date won't be changed
		// if this parameter is given.
		if (!strcmp(p, "-h")) {
			printf("Usage: SetDate [YYYY-MM-DD|YY-MM-DD|HH:MM:SS] [-h]\n"
				"YYYY-MM-DD\tSet date, e.g. 2015-09-18\n"
				"YY-MM-DD\tSet date, e.g. 15-09-18\n"
				"HH:MM:SS\tSet time, e.g. 12:34:56\n"
				"HH:MM\t\tSet time, e.g. 12:34\n"
				"-h\t\tShow this help\n");
			return S_OK;
		
		// Each of the four different input formats are in their own statements
		// Multiple formats can be given with one program execution.
		} else {
			
			// YYYY-MM-DD
			if (strlen(p) == 10 && p[4] == '-' && p[7] == '-')  {
				tm->tm_year = atoi(p+0)-1900;
				tm->tm_mon = atoi(p+5)-1;
				tm->tm_mday = atoi(p+8);
				set |= 1;
				
			// YY-MM-DD
			} else if (strlen(p) == 8 && p[2] == '-' && p[5] == '-')  {
				tm->tm_year = atoi(p+0)+100;
				tm->tm_mon = atoi(p+3)-1;
				tm->tm_mday = atoi(p+6);
				set |= 1;
				
			// HH:MM:SS
			} else if (strlen(p) == 8 && p[2] == ':' && p[5] == ':')  {
				tm->tm_hour = atoi(p+0);
				tm->tm_min = atoi(p+3);
				tm->tm_sec = atoi(p+6);
				set |= 2;
				
			// HH:MM
			} else if (strlen(p) == 5 && p[2] == ':')  {
				tm->tm_hour = atoi(p+0);	
				tm->tm_min = atoi(p+3);
				tm->tm_sec = 0;
				set |= 2;
				
			// Incorrect format
			} else {
				printf("E: Malformed parameter \"%s\"\n", p);
				res = S_ERROR;
				goto finally;	
			}
		}
		p += strlen(p)+1;
	}

	// set is non-zero if a correctly formatted date or time was given
	if (set) {
		
		// mktime() converts the time from the tm struct to seconds, which 
		// can be used by rtc.
		u_int32 t = (u_int32)mktime(tm);
		
		// If time and date and date haven't been set yet, the time() function
		// will return a negative value. If both values were given as input,
		// the set variable will have the value 3.
		if (tim < 0 && set < 3) {
			printf("E: RTC not set. You need to set both date and time!\n");
			res = S_ERROR;
		
		// If no error, set the RTC value.
		} else {
			SetRtc(t);
 	   }
	}

	finally:
		return res;
}


































